# Generated from FullForm.g4 by ANTLR 4.7.2
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .FullFormParser import FullFormParser
else:
    from FullFormParser import FullFormParser

# This class defines a complete listener for a parse tree produced by FullFormParser.
class FullFormListener(ParseTreeListener):

    # Enter a parse tree produced by FullFormParser#prog.
    def enterProg(self, ctx:FullFormParser.ProgContext):
        pass

    # Exit a parse tree produced by FullFormParser#prog.
    def exitProg(self, ctx:FullFormParser.ProgContext):
        pass


    # Enter a parse tree produced by FullFormParser#Number.
    def enterNumber(self, ctx:FullFormParser.NumberContext):
        pass

    # Exit a parse tree produced by FullFormParser#Number.
    def exitNumber(self, ctx:FullFormParser.NumberContext):
        pass


    # Enter a parse tree produced by FullFormParser#StringLiteral.
    def enterStringLiteral(self, ctx:FullFormParser.StringLiteralContext):
        pass

    # Exit a parse tree produced by FullFormParser#StringLiteral.
    def exitStringLiteral(self, ctx:FullFormParser.StringLiteralContext):
        pass


    # Enter a parse tree produced by FullFormParser#SymbolLiteral.
    def enterSymbolLiteral(self, ctx:FullFormParser.SymbolLiteralContext):
        pass

    # Exit a parse tree produced by FullFormParser#SymbolLiteral.
    def exitSymbolLiteral(self, ctx:FullFormParser.SymbolLiteralContext):
        pass


    # Enter a parse tree produced by FullFormParser#HeadExpression.
    def enterHeadExpression(self, ctx:FullFormParser.HeadExpressionContext):
        pass

    # Exit a parse tree produced by FullFormParser#HeadExpression.
    def exitHeadExpression(self, ctx:FullFormParser.HeadExpressionContext):
        pass


    # Enter a parse tree produced by FullFormParser#ExpressionListed.
    def enterExpressionListed(self, ctx:FullFormParser.ExpressionListedContext):
        pass

    # Exit a parse tree produced by FullFormParser#ExpressionListed.
    def exitExpressionListed(self, ctx:FullFormParser.ExpressionListedContext):
        pass


    # Enter a parse tree produced by FullFormParser#ContextName.
    def enterContextName(self, ctx:FullFormParser.ContextNameContext):
        pass

    # Exit a parse tree produced by FullFormParser#ContextName.
    def exitContextName(self, ctx:FullFormParser.ContextNameContext):
        pass


    # Enter a parse tree produced by FullFormParser#SimpleContext.
    def enterSimpleContext(self, ctx:FullFormParser.SimpleContextContext):
        pass

    # Exit a parse tree produced by FullFormParser#SimpleContext.
    def exitSimpleContext(self, ctx:FullFormParser.SimpleContextContext):
        pass


    # Enter a parse tree produced by FullFormParser#CompoundContext.
    def enterCompoundContext(self, ctx:FullFormParser.CompoundContextContext):
        pass

    # Exit a parse tree produced by FullFormParser#CompoundContext.
    def exitCompoundContext(self, ctx:FullFormParser.CompoundContextContext):
        pass


    # Enter a parse tree produced by FullFormParser#NumberBaseN.
    def enterNumberBaseN(self, ctx:FullFormParser.NumberBaseNContext):
        pass

    # Exit a parse tree produced by FullFormParser#NumberBaseN.
    def exitNumberBaseN(self, ctx:FullFormParser.NumberBaseNContext):
        pass


    # Enter a parse tree produced by FullFormParser#NumberBaseTen.
    def enterNumberBaseTen(self, ctx:FullFormParser.NumberBaseTenContext):
        pass

    # Exit a parse tree produced by FullFormParser#NumberBaseTen.
    def exitNumberBaseTen(self, ctx:FullFormParser.NumberBaseTenContext):
        pass


    # Enter a parse tree produced by FullFormParser#numberLiteralPrecision.
    def enterNumberLiteralPrecision(self, ctx:FullFormParser.NumberLiteralPrecisionContext):
        pass

    # Exit a parse tree produced by FullFormParser#numberLiteralPrecision.
    def exitNumberLiteralPrecision(self, ctx:FullFormParser.NumberLiteralPrecisionContext):
        pass


    # Enter a parse tree produced by FullFormParser#numberLiteralExponent.
    def enterNumberLiteralExponent(self, ctx:FullFormParser.NumberLiteralExponentContext):
        pass

    # Exit a parse tree produced by FullFormParser#numberLiteralExponent.
    def exitNumberLiteralExponent(self, ctx:FullFormParser.NumberLiteralExponentContext):
        pass


